function DAVIFeatures1 = fn_DAVIFeatureExtraction1(DispMap)
%% input is Disparity map, which is a result of Optical flow

%Disparity(pixel)
d_x = DispMap; 

%% Parameters
WSIZE = 1920;
HSIZE = 1080;
WIDTH = WSIZE;
HEIGHT = HSIZE;
l_x = 1.016; %  Horizontal size (m)
l_y = 0.5715; %  Vertical size (m)
N_x = WIDTH; % Horizontal resolution (pixel)
N_y = HEIGHT; % Vertical resolution (pixel)
l_e = 0.06; % inter-pupil distance (m)

%% Angular disparity calculaton
v(1) = l_x/2;
v(2) = l_y/2;
v(3) = 1.7145;

% Coordinate of eyes of audiance
L = [v(1)-l_e/2,v(2),v(3)]; % left eye
R = [v(1)+l_e/2,v(2),v(3)]; % right eye

% Pixel Length coordinate
L_disp_x = zeros(HEIGHT, WIDTH);
for n_x = 1:WIDTH
    L_disp_x(:,n_x) = n_x*(l_x/N_x);
end
L_disp_y = zeros(HEIGHT, WIDTH);
for n_y = 1:HEIGHT
    L_disp_y(HEIGHT-n_y+1,:) = n_y*(l_y/N_y);
end
L_disp_z = zeros(HEIGHT, WIDTH);

P_x = L_disp_x;
P_y = L_disp_y;
P_z = L_disp_z;
Pd_x = zeros(HEIGHT,WIDTH);
for n_y = 1:HEIGHT
    for n_x = 1:WIDTH
        Pd_x(n_y,n_x) = (n_x+d_x(n_y,n_x))*(l_x/N_x);
    end
end
Pd_y = L_disp_y;
Pd_z = L_disp_z;

alpha_d_tmp2 = zeros(HEIGHT,WIDTH);
alpha_d_tmp2(:,:) = acos(((R(1)-P_x(:,:)).^2+(R(2)-P_y(:,:)).^2+(R(3)-P_z(:,:)).^2 + (R(1)-Pd_x(:,:)).^2+(R(2)-Pd_y(:,:)).^2+(R(3)-Pd_z(:,:)).^2 ...
    - (P_x(:,:)-Pd_x(:,:)).^2+(P_y(:,:)-Pd_y(:,:)).^2+(P_z(:,:)-Pd_z(:,:)).^2) ... 
    ./(2 * sqrt((R(1)-P_x(:,:)).^2+(R(2)-P_y(:,:)).^2+(R(3)-P_z(:,:)).^2) .* sqrt((R(1)-Pd_x(:,:)).^2+(R(2)-Pd_y(:,:)).^2+(R(3)-Pd_z(:,:)).^2)));

alpha_d_sign = zeros(HEIGHT,WIDTH);
alpha_d_crossed = zeros(HEIGHT,WIDTH);
alpha_d_uncrossed = zeros(HEIGHT,WIDTH);
alpha_d_sign(:,:) = (d_x(:,:)<0); % 1 means negative disparity,0 means positive disparity
alpha_d = ((180/pi)*alpha_d_tmp2);
alpha_d_crossed = alpha_d.*alpha_d_sign;
alpha_d_uncrossed = alpha_d.*abs(alpha_d_sign-1);

%% Weight
a = alpha_d_tmp2; %radian
a_d = alpha_d; %Degree
vv = sqrt((v(1)-L_disp_x).*(v(1)-L_disp_x) + (v(2)-L_disp_y).*(v(2)-L_disp_y) + (v(3)-L_disp_z).*(v(3)-L_disp_z));

d = a.*(vv.*vv)/l_e;

d = abs(d);
a = abs(a);
a_d = abs(a_d);

DEPTH_near = alpha_d_sign;
DEPTH_far = abs(alpha_d_sign-1);

%% Transfer function
TF_acc = tf([5 21 4], [1 15.57 66.77 75.04 12.46]); % accommodation  transfer function
TF_ver = tf([3 22.7 42.72 7.66], [1 15.57 66.77 75.04 12.46]); % vergence  transfer function

AC_Map = zeros(HSIZE,WSIZE);
AA_Map = zeros(HSIZE,WSIZE);
VV_Map = zeros(HSIZE,WSIZE);
Object_z = alpha_d;
count = 0;
for i = 1:100
    for j = 1:10
        count = count+1;
        i
        a = v(1)*step(TF_acc);
        b = Object_z(i,j)*step(TF_ver);
        c = max(a)+max(b);  
        AA_Map(i,j) = max(a)+max(a);
        VV_Map(i,j) = max(b)+max(b);
        AC_Map(i,j) = c;
    end
end
clear count
load ACresponse; % Step Response of Transfer function  step ҷ

REA_ver_map1 = a_d.*AC_Map + a_d.*VV_Map; % Real϶ vergence 
STE_ver_map1  = 0.*AC_Map + a_d.*VV_Map; % ACC ũ   vergence 
CON_ver_map1 = REA_ver_map1 - STE_ver_map1; %   (Confliction) 

REA_acc_map1 = a_d.*AA_Map + a_d.*AC_Map; % Real϶ vergence 
STE_acc_map1  = 0.*AA_Map + a_d.*AC_Map; % ACC ũ   vergence 
CON_acc_map1 = REA_acc_map1 - STE_acc_map1; %   (Confliction) 

CON_acc_ver_map1 = CON_ver_map1 - CON_acc_map1;

REA_ver_map = a_d.*RES_acc_ver + a_d.*RES_ver_ver; % Real϶ vergence 
STE_ver_map  = 0.*RES_acc_ver + a_d.*RES_ver_ver; % ACC ũ   vergence 
CON_ver_map = REA_ver_map - STE_ver_map; %   (Confliction) 

REA_acc_map = a_d.*RES_acc_acc + a_d.*RES_ver_acc; % Real϶ vergence 
STE_acc_map  = 0.*RES_acc_acc + a_d.*RES_ver_acc; % ACC ũ   vergence 
CON_acc_map = REA_acc_map - STE_acc_map; %   (Confliction) 

CON_acc_ver_map = CON_ver_map - CON_acc_map;


%% Feature extraction
PERCENT = 5;
Bin = 1000; %HSIZE*WSIZE

[REA_ver_mean_near, REA_ver_mean_near_h, REA_ver_mean_near_l] = HistoCal(REA_ver_map, (DEPTH_near>0), PERCENT);
[REA_ver_mean_far, REA_ver_mean_far_h, REA_ver_mean_far_l] = HistoCal(REA_ver_map, (DEPTH_far>0), PERCENT);
[STE_ver_mean_near, STE_ver_mean_near_h, STE_ver_mean_near_l] = HistoCal(STE_ver_map, (DEPTH_near>0), PERCENT);
[STE_ver_mean_far, STE_ver_mean_far_h, STE_ver_mean_far_l] = HistoCal(STE_ver_map, (DEPTH_far>0), PERCENT);
[CON_ver_mean_near, CON_ver_mean_near_h, CON_ver_mean_near_l] = HistoCal(CON_ver_map, (DEPTH_near>0), PERCENT);
[CON_ver_mean_far, CON_ver_mean_far_h, CON_ver_mean_far_l] = HistoCal(CON_ver_map, (DEPTH_far>0), PERCENT);

[REA_acc_mean_near, REA_acc_mean_near_h, REA_acc_mean_near_l] = HistoCal(REA_acc_map, (DEPTH_near>0), PERCENT);
[REA_acc_mean_far, REA_acc_mean_far_h, REA_acc_mean_far_l] = HistoCal(REA_acc_map, (DEPTH_far>0), PERCENT);
[STE_acc_mean_near, STE_acc_mean_near_h, STE_acc_mean_near_l] = HistoCal(STE_acc_map, (DEPTH_near>0), PERCENT);
[STE_acc_mean_far, STE_acc_mean_far_h, STE_acc_mean_far_l] = HistoCal(STE_acc_map, (DEPTH_far>0), PERCENT);
[CON_acc_mean_near, CON_acc_mean_near_h, CON_acc_mean_near_l] = HistoCal(CON_acc_map, (DEPTH_near>0), PERCENT);
[CON_acc_mean_far, CON_acc_mean_far_h, CON_acc_mean_far_l] = HistoCal(CON_acc_map, (DEPTH_far>0), PERCENT);

[CON_acc_ver_mean_near, CON_acc_ver_mean_near_h, CON_acc_ver_mean_near_l] = HistoCal(CON_acc_ver_map, (DEPTH_near>0), PERCENT);
[CON_acc_ver_mean_far, CON_acc_ver_mean_far_h, CON_acc_ver_mean_far_l] = HistoCal(CON_acc_ver_map, (DEPTH_far>0), PERCENT);

if (STE_acc_mean_far == 0);
    STE_acc_rel = 0;
else
    STE_acc_rel = abs(STE_acc_mean_near/STE_acc_mean_far);
end
if (STE_ver_mean_far == 0);
    STE_ver_rel = 0;
else
    STE_ver_rel = abs(STE_ver_mean_near/STE_ver_mean_far);
end

DAVIFeatures1 = [CON_acc_ver_mean_near, CON_acc_ver_mean_near_h, CON_acc_ver_mean_far, CON_acc_ver_mean_far_h, STE_acc_rel, STE_ver_rel];

end