function DAVIFeatures2 = fn_DAVIFeatureExtraction2(DispMap)
%% input is Disparity map, which is a result of Optical flow

%Disparity(pixel)
d_x = DispMap;

%% Parameters
WSIZE = 1920;
HSIZE = 1080;
WIDTH = WSIZE;
HEIGHT = HSIZE;
l_x = 1.016; %  Horizontal size (m)
l_y = 0.5715; %  Vertical size (m)
N_x = WIDTH; % Horizontal resolution (pixel)
N_y = HEIGHT; % Vertical resolution (pixel)
l_e = 0.06; % inter-pupil distance (m)

%% Angular disparity calculaton
v(1) = l_x/2;
v(2) = l_y/2;
v(3) = 1.7145;

% Coordinate of eyes of audiance
L = [v(1)-l_e/2,v(2),v(3)]; % left eye
R = [v(1)+l_e/2,v(2),v(3)]; % right eye

% Pixel Length coordinate
L_disp_x = zeros(HEIGHT, WIDTH);
for n_x = 1:WIDTH
    L_disp_x(:,n_x) = n_x*(l_x/N_x);
end
L_disp_y = zeros(HEIGHT, WIDTH);
for n_y = 1:HEIGHT
    L_disp_y(HEIGHT-n_y+1,:) = n_y*(l_y/N_y);
end
L_disp_z = zeros(HEIGHT, WIDTH);

P_x = L_disp_x;   
P_y = L_disp_y;
P_z = L_disp_z;
Pd_x = zeros(HEIGHT,WIDTH);
for n_y = 1:HEIGHT
    for n_x = 1:WIDTH
        Pd_x(n_y,n_x) = (n_x+d_x(n_y,n_x))*(l_x/N_x);
    end
end
Pd_y = L_disp_y;
Pd_z = L_disp_z;

alpha_d_tmp2 = zeros(HEIGHT,WIDTH);
alpha_d_tmp2(:,:) = acos(((R(1)-P_x(:,:)).^2+(R(2)-P_y(:,:)).^2+(R(3)-P_z(:,:)).^2 + (R(1)-Pd_x(:,:)).^2+(R(2)-Pd_y(:,:)).^2+(R(3)-Pd_z(:,:)).^2 ...
    - (P_x(:,:)-Pd_x(:,:)).^2+(P_y(:,:)-Pd_y(:,:)).^2+(P_z(:,:)-Pd_z(:,:)).^2) ... 
    ./(2 * sqrt((R(1)-P_x(:,:)).^2+(R(2)-P_y(:,:)).^2+(R(3)-P_z(:,:)).^2) .* sqrt((R(1)-Pd_x(:,:)).^2+(R(2)-Pd_y(:,:)).^2+(R(3)-Pd_z(:,:)).^2)));

alpha_d_sign = zeros(HEIGHT,WIDTH);
alpha_d_crossed = zeros(HEIGHT,WIDTH);
alpha_d_uncrossed = zeros(HEIGHT,WIDTH);

alpha_d_sign(:,:) = (d_x(:,:)<0); 
alpha_d = ((180/pi)*alpha_d_tmp2);
alpha_d_crossed = alpha_d.*alpha_d_sign;
alpha_d_uncrossed = alpha_d.*abs(alpha_d_sign-1);

THETA_d(:,:) = alpha_d; % Angular disparity

%% Panum's Fusion Area

Thres = 0;
Decrement = 0.31;

FUSION_map = exp(-(THETA_d-Thres)./Decrement); %For faster calculation

%% Pentland's DOF
d = abs(alpha_d_tmp2).*(v(3).*v(3))/l_e;
Object_z = zeros(HSIZE,WSIZE);
DEPTH_near = alpha_d_sign;
DEPTH_far = abs(alpha_d_sign-1);
for i=1:HSIZE
    for j=1:WSIZE
        if alpha_d_sign(i,j)==1
            Object_z(i,j) = v(3)-d(i,j); % negative disparity
        else
            Object_z(i,j) = v(3)+d(i,j); % positive disparity
        end
    end
end
r = 4.6/1000;
s_0 = 10/1000;
SIGMA_map = 100000*r*s_0*abs(Object_z-v(3))./(Object_z*v(3));

%% Variance
FUSION_map_1D = reshape(FUSION_map,1,HSIZE*WSIZE);
SIGMA_map_1D = reshape(SIGMA_map,1,HSIZE*WSIZE);

%% Feature extraction
PERCENT = 5;
Bin = 1000;%HSIZE*WSIZE

[FUSION_mean_near, FUSION_mean_near_h, FUSION_mean_near_l] = HistoCal(FUSION_map, (DEPTH_near>0), PERCENT);
[FUSION_mean_far, FUSION_mean_far_h, FUSION_mean_far_l] = HistoCal(FUSION_map, (DEPTH_far>0), PERCENT);
FUSION_var = var(FUSION_map_1D);

[SIGMA_mean_near, SIGMA_mean_near_h, SIGMA_mean_near_l] = HistoCal(SIGMA_map, (DEPTH_near>0), PERCENT);
[SIGMA_mean_far, SIGMA_mean_far_h, SIGMA_mean_far_l] = HistoCal(SIGMA_map, (DEPTH_far>0), PERCENT);
SIGMA_var = var(SIGMA_map_1D);

DAVIFeatures2 = [FUSION_mean_near, FUSION_mean_far, FUSION_mean_near_l, FUSION_mean_far_l, FUSION_var, SIGMA_mean_near, SIGMA_mean_far, SIGMA_mean_near_l, SIGMA_mean_far_l];

end